/*-------------------------------------------*/
/*   Sample code(1) of using SSI32 on MPI    */
/*   (simply generate successive mpisize*10  */
/*    random numbers)                        */
/*-------------------------------------------*/

#define useMPI    // if not use MPI, remove this line
#define Use_icc   // if your compiler is icc, use this line

#define mpisizeMax         1024
#define RNsInOneMPIProcess 10

#include <stdio.h>
#ifdef useMPI
#include <mpi.h>   /*MPI*/
#endif

#include "SSI32rand.c"

/*******************/

#ifdef useMPI
int main(int argc, char * argv[])
#else
int main()
#endif
    {
    int i;

    #ifdef useMPI
    MPI_Status mpistat;   /*MPI*/
    #endif
    int  mpierr=0;
    int  mpisize=1, myrank=0;
    int  RecvNo;
    unsigned int randVal[RNsInOneMPIProcess*mpisizeMax];

    #ifdef useMPI
    MPI_Init(&argc, &argv);                   /*MPI*/
    MPI_Comm_size(MPI_COMM_WORLD, &mpisize);  /*MPI*/
    MPI_Comm_rank(MPI_COMM_WORLD, &myrank);   /*MPI*/
    #endif
    if (myrank==0)
        {
        printf("mpisize = %d\n", mpisize);
        if (mpisize>mpisizeMax) {printf("Error: mpisize>mpisizeMax\n");}
        }
    if (mpisize>mpisizeMax)
        {
        #ifdef useMPI
        MPI_Finalize();   /*MPI*/
        #endif
        return(1);
        }

    SSI32IniK(RNsInOneMPIProcess*myrank);       //set the position of generating RNs
    for (i=0; i<RNsInOneMPIProcess; i++)
        {randVal[RNsInOneMPIProcess*myrank+i]=SSI32KBin32();}  //get a random number

    if (myrank==0)
        {
        for (RecvNo=1; RecvNo<mpisize; RecvNo++)
            {
            #ifdef useMPI
            mpierr=MPI_Recv(&randVal[RNsInOneMPIProcess*RecvNo], RNsInOneMPIProcess,
                             MPI_INT, RecvNo, 1234, MPI_COMM_WORLD, &mpistat);  /*MPI*/
            #endif
            if (mpierr) {printf("mpi recv err : myrank=%d\n", RecvNo);}
            }
        }
      else
        {
        #ifdef useMPI
        mpierr=MPI_Send(&randVal[RNsInOneMPIProcess*myrank], RNsInOneMPIProcess,
                         MPI_INT, 0, 1234, MPI_COMM_WORLD);                    /*MPI*/
        #endif
        if (mpierr) {printf("mpi send err : %d\n", RecvNo);}
        }
    #ifdef useMPI
    MPI_Finalize();   /*MPI*/
    #endif

    if (myrank==0)
        {
        printf("SSI32-random-numbers from  0th  to %dth are\n", RNsInOneMPIProcess*mpisize-1);
        for (i=0; i<RNsInOneMPIProcess*mpisize; i++)
            {printf("[%d] %08x\n", i, randVal[i]);}
        /* The result is
            [0] 76d44f81   [1] aac718ec   [2] 9d91efc0   [3] 4b043dfa   [4] ebef2efd
            [5] 8a753122   [6] 82227f2f   [7] 42440bbc   [8] 8c7e95d4   [9] cb527581
            [10] 5f67c34c  [11] 35d09850  [12] eec4b498  [13] 77997371   ...      */
        }

    return(0);
    }

